@@ -8,6 +8,7 @@
 #include <bovy_coords.h>
 #include <bovy_symplecticode.h>
 #include <bovy_rk.h>
+#include <wez_ias15.h>
 #include <leung_dop853.h>
 #include <integrateFullOrbit.h>
 //Potentials
@@ -34,6 +35,10 @@ void evalPlanarRectForce(double, double *, double *,
 			 int, struct potentialArg *);
 void evalPlanarRectDeriv(double, double *, double *,
 			 int, struct potentialArg *);
+void evalPlanarSOSDerivx(double, double *, double *,
+			 int, struct potentialArg *);
+void evalPlanarSOSDerivy(double, double *, double *,
+			 int, struct potentialArg *);
 void evalPlanarRectDeriv_dxdv(double, double *, double *,
 			      int, struct potentialArg *);
 void initPlanarMovingObjectSplines(struct potentialArg *, double ** pot_args);
@@ -58,6 +63,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->planarRphideriv= &LogarithmicHaloPotentialPlanarRphideriv;
       potentialArgs->nargs= 4;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 1: //DehnenBarPotential, 6 arguments
       potentialArgs->planarRforce= &DehnenBarPotentialPlanarRforce;
@@ -67,18 +73,21 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->planarRphideriv= &DehnenBarPotentialPlanarRphideriv;
       potentialArgs->nargs= 6;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 2: //TransientLogSpiralPotential, 8 arguments
       potentialArgs->planarRforce= &TransientLogSpiralPotentialRforce;
       potentialArgs->planarphitorque= &TransientLogSpiralPotentialphitorque;
       potentialArgs->nargs= 8;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 3: //SteadyLogSpiralPotential, 8 arguments
       potentialArgs->planarRforce= &SteadyLogSpiralPotentialRforce;
       potentialArgs->planarphitorque= &SteadyLogSpiralPotentialphitorque;
       potentialArgs->nargs= 8;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 4: //EllipticalDiskPotential, 6 arguments
       potentialArgs->planarRforce= &EllipticalDiskPotentialRforce;
@@ -88,6 +97,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->planarRphideriv= &EllipticalDiskPotentialRphideriv;
       potentialArgs->nargs= 6;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 5: //MiyamotoNagaiPotential, 3 arguments
       potentialArgs->potentialEval= &MiyamotoNagaiPotentialEval;
@@ -98,6 +108,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->planarRphideriv= &ZeroPlanarForce;
       potentialArgs->nargs= 3;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 6: //LopsidedDiskPotential, 4 arguments
       potentialArgs->planarRforce= &LopsidedDiskPotentialRforce;
@@ -107,6 +118,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->planarRphideriv= &LopsidedDiskPotentialRphideriv;
       potentialArgs->nargs= 4;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 7: //PowerSphericalPotential, 2 arguments
       potentialArgs->potentialEval= &PowerSphericalPotentialEval;
@@ -117,6 +129,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->planarRphideriv= &ZeroPlanarForce;
       potentialArgs->nargs= 2;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 8: //HernquistPotential, 2 arguments
       potentialArgs->potentialEval= &HernquistPotentialEval;
@@ -127,6 +140,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->planarRphideriv= &ZeroPlanarForce;
       potentialArgs->nargs= 2;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 9: //NFWPotential, 2 arguments
       potentialArgs->potentialEval= &NFWPotentialEval;
@@ -137,6 +151,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->planarRphideriv= &ZeroPlanarForce;
       potentialArgs->nargs= 2;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 10: //JaffePotential, 2 arguments
       potentialArgs->potentialEval= &JaffePotentialEval;
@@ -147,6 +162,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->planarRphideriv= &ZeroPlanarForce;
       potentialArgs->nargs= 2;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 11: //DoubleExponentialDiskPotential, XX arguments
       potentialArgs->potentialEval= &DoubleExponentialDiskPotentialEval;
@@ -158,6 +174,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       //Look at pot_args to figure out the number of arguments
       potentialArgs->nargs= (int) (5 + 4 * *(*pot_args+4) );
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 12: //FlattenedPowerPotential, 4 arguments
       potentialArgs->potentialEval= &FlattenedPowerPotentialEval;
@@ -168,6 +185,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->planarRphideriv= &ZeroPlanarForce;
       potentialArgs->nargs= 3;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 14: //IsochronePotential, 2 arguments
       potentialArgs->potentialEval= &IsochronePotentialEval;
@@ -178,6 +196,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->planarRphideriv= &ZeroPlanarForce;
       potentialArgs->nargs= 2;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 15: //PowerSphericalPotentialwCutoff, 3 arguments
       potentialArgs->potentialEval= &PowerSphericalPotentialwCutoffEval;
@@ -188,6 +207,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->planarRphideriv= &ZeroPlanarForce;
       potentialArgs->nargs= 3;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 16: //KuzminKutuzovStaeckelPotential, 3 arguments
       potentialArgs->potentialEval= &KuzminKutuzovStaeckelPotentialEval;
@@ -198,6 +218,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->planarRphideriv= &ZeroPlanarForce;
       potentialArgs->nargs= 3;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 17: //PlummerPotential, 2 arguments
       potentialArgs->potentialEval= &PlummerPotentialEval;
@@ -208,6 +229,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->planarRphideriv= &ZeroPlanarForce;
       potentialArgs->nargs= 2;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 18: //PseudoIsothermalPotential, 2 arguments
       potentialArgs->potentialEval= &PseudoIsothermalPotentialEval;
@@ -218,6 +240,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->planarRphideriv= &ZeroPlanarForce;
       potentialArgs->nargs= 2;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 19: //KuzminDiskPotential, 2 arguments
       potentialArgs->potentialEval= &KuzminDiskPotentialEval;
@@ -228,6 +251,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->planarRphideriv= &ZeroPlanarForce;
       potentialArgs->nargs= 2;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 20: //BurkertPotential, 2 arguments
       potentialArgs->potentialEval= &BurkertPotentialEval;
@@ -238,6 +262,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->planarRphideriv= &ZeroPlanarForce;
       potentialArgs->nargs= 2;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 21: // TriaxialHernquistPotential, lots of arguments
       potentialArgs->planarRforce = &EllipsoidalPotentialPlanarRforce;
@@ -252,6 +277,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->nargs = (int) (21 + *(*pot_args+7) + 2 * *(*pot_args
 					    + (int) (*(*pot_args+7) + 20)));
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 22: // TriaxialNFWPotential, lots of arguments
       potentialArgs->planarRforce = &EllipsoidalPotentialPlanarRforce;
@@ -266,6 +292,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->nargs = (int) (21 + *(*pot_args+7) + 2 * *(*pot_args
 					    + (int) (*(*pot_args+7) + 20)));
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 23: // TriaxialJaffePotential, lots of arguments
       potentialArgs->planarRforce = &EllipsoidalPotentialPlanarRforce;
@@ -280,6 +307,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->nargs = (int) (21 + *(*pot_args+7) + 2 * *(*pot_args
 					    + (int) (*(*pot_args+7) + 20)));
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 24: //SCFPotential, many arguments
       potentialArgs->potentialEval= &SCFPotentialEval;
@@ -290,6 +318,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->planarRphideriv= &SCFPotentialPlanarRphideriv;
       potentialArgs->nargs= (int) (5 + (1 + *(*pot_args + 1)) * *(*pot_args+2) * *(*pot_args+3)* *(*pot_args+4) + 7);
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 25: //SoftenedNeedleBarPotential, 13 arguments
       potentialArgs->potentialEval= &SoftenedNeedleBarPotentialEval;
@@ -297,6 +326,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->planarphitorque= &SoftenedNeedleBarPotentialPlanarphitorque;
       potentialArgs->nargs= 13;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 26: //DiskSCFPotential, nsigma+3 arguments
       potentialArgs->potentialEval= &DiskSCFPotentialEval;
@@ -304,6 +334,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->planarphitorque= &ZeroPlanarForce;
       potentialArgs->nargs= (int) **pot_args + 3;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 27: // SpiralArmsPotential, 10 arguments + array of Cs
       potentialArgs->planarRforce = &SpiralArmsPotentialPlanarRforce;
@@ -313,6 +344,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->planarRphideriv = &SpiralArmsPotentialPlanarRphideriv;
       potentialArgs->nargs = (int) 10 + **pot_args;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 28: //CosmphiDiskPotential, 9 arguments
       potentialArgs->planarRforce= &CosmphiDiskPotentialRforce;
@@ -322,6 +354,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->planarRphideriv= &CosmphiDiskPotentialRphideriv;
       potentialArgs->nargs= 9;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 29: //HenonHeilesPotential, 1 argument
       potentialArgs->planarRforce= &HenonHeilesPotentialRforce;
@@ -331,6 +364,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->planarRphideriv= &HenonHeilesPotentialRphideriv;
       potentialArgs->nargs= 1;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 30: // PerfectEllipsoidPotential, lots of arguments
       potentialArgs->planarRforce = &EllipsoidalPotentialPlanarRforce;
@@ -345,6 +379,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->nargs = (int) (21 + *(*pot_args+7) + 2 * *(*pot_args
 					    + (int) (*(*pot_args+7) + 20)));
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     // 31: KGPotential
     // 32: IsothermalDiskPotential
@@ -357,6 +392,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->planarRphideriv= &ZeroPlanarForce;
       potentialArgs->nargs= 2;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 34: //DehnenSphericalpotential
       potentialArgs->potentialEval= &DehnenSphericalPotentialEval;
@@ -367,6 +403,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->planarRphideriv= &ZeroPlanarForce;
       potentialArgs->nargs= 3;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 35: //HomogeneousSpherePotential, 3 arguments
       potentialArgs->potentialEval= &HomogeneousSpherePotentialEval;
@@ -377,6 +414,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->planarRphideriv= &ZeroPlanarForce;
       potentialArgs->nargs= 3;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 36: //interpSphericalPotential, XX arguments
       // Set up 1 spline in potentialArgs
@@ -420,6 +458,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->nargs = (int) (21 + *(*pot_args+7) + 2 * *(*pot_args
 					    + (int) (*(*pot_args+7) + 20)));
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case 38: // PowerTriaxialPotential, lots of arguments
       potentialArgs->planarRforce = &EllipsoidalPotentialPlanarRforce;
@@ -434,6 +473,15 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->nargs = (int) (21 + *(*pot_args+7) + 2 * *(*pot_args
 					    + (int) (*(*pot_args+7) + 20)));
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
+      break;
+    case 39: //NonInertialFrameForce, 22 arguments (10 caching ones)
+      potentialArgs->planarRforceVelocity= &NonInertialFrameForcePlanarRforce;
+      potentialArgs->planarphitorqueVelocity= &NonInertialFrameForcePlanarphitorque;
+      potentialArgs->nargs= 23;
+      potentialArgs->ntfuncs= (int) ( 3 * *(*pot_args + 12) * ( 1 + 2 * *(*pot_args + 11) ) \
+                                + ( 6 - 4 * ( *(*pot_args + 13) ) ) * *(*pot_args + 15) );
+      potentialArgs->requiresVelocity= true;
       break;
     case 40: //NullPotential, no arguments (only supported for orbit int)
       potentialArgs->potentialEval= &ZeroForce;
@@ -444,6 +492,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->planarRphideriv= &ZeroPlanarForce;
       potentialArgs->nargs= 0;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
 //////////////////////////////// WRAPPERS /////////////////////////////////////
     case -1: //DehnenSmoothWrapperPotential
@@ -455,6 +504,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->planarRphideriv= &DehnenSmoothWrapperPotentialPlanarRphideriv;
       potentialArgs->nargs= 4;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case -2: //SolidBodyRotationWrapperPotential
       potentialArgs->planarRforce= &SolidBodyRotationWrapperPotentialPlanarRforce;
@@ -464,6 +514,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->planarRphideriv= &SolidBodyRotationWrapperPotentialPlanarRphideriv;
       potentialArgs->nargs= 3;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case -4: //CorotatingRotationWrapperPotential
       potentialArgs->planarRforce= &CorotatingRotationWrapperPotentialPlanarRforce;
@@ -473,6 +524,7 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->planarRphideriv= &CorotatingRotationWrapperPotentialPlanarRphideriv;
       potentialArgs->nargs= 5;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case -5: //GaussianAmplitudeWrapperPotential
       potentialArgs->planarRforce= &GaussianAmplitudeWrapperPotentialPlanarRforce;
@@ -482,12 +534,14 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->planarRphideriv= &GaussianAmplitudeWrapperPotentialPlanarRphideriv;
       potentialArgs->nargs= 3;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     case -6: //MovingObjectPotential
       potentialArgs->planarRforce= &MovingObjectPotentialPlanarRforce;
       potentialArgs->planarphitorque= &MovingObjectPotentialPlanarphitorque;
       potentialArgs->nargs= 3;
       potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     //ChandrasekharDynamicalFrictionForce omitted, bc no planar version
     //RotateAndTiltWrapperPotential omitted, bc no planar version
@@ -500,6 +554,18 @@ void parse_leapFuncArgs(int npot,struct potentialArg * potentialArgs,
       potentialArgs->planarRphideriv= &TimeDependentAmplitudeWrapperPotentialPlanarRphideriv;
       potentialArgs->nargs= 4;
       potentialArgs->ntfuncs= 1;
+      potentialArgs->requiresVelocity= false;
+      break;
+    case -10: //KuzminLikeWrapperPotential
+      potentialArgs->potentialEval= &KuzminLikeWrapperPotentialEval;
+      potentialArgs->planarRforce= &KuzminLikeWrapperPotentialPlanarRforce;
+      potentialArgs->planarphitorque= &ZeroPlanarForce;
+      potentialArgs->planarR2deriv= &KuzminLikeWrapperPotentialPlanarR2deriv;
+      potentialArgs->planarphi2deriv= &ZeroPlanarForce;
+      potentialArgs->planarRphideriv= &ZeroPlanarForce;
+      potentialArgs->nargs= 3;
+      potentialArgs->ntfuncs= 0;
+      potentialArgs->requiresVelocity= false;
       break;
     }
     int setupSplines = *(*pot_type-1) == -6 ? 1 : 0;
@@ -609,6 +675,11 @@ EXPORT void integratePlanarOrbit(int nobj,
     odeint_deriv_func= &evalPlanarRectDeriv;
     dim= 4;
     break;
+  case 7: //ias15
+    odeint_func= &wez_ias15;
+    odeint_deriv_func= &evalPlanarRectForce;
+    dim= 2;
+    break;
   }
 #pragma omp parallel for schedule(dynamic,ORBITS_CHUNKSIZE) private(ii,jj) num_threads(max_threads)
   for (ii=0; ii < nobj; ii++) {
@@ -628,7 +699,98 @@ EXPORT void integratePlanarOrbit(int nobj,
   free(potentialArgs);
   //Done!
 }
+EXPORT void integratePlanarOrbit_sos(
+    int nobj,
+	double *yo,
+	int npsi,
+	double *psi,
+    int indiv_psi,
+    int surface,
+    int npot,
+	int * pot_type,
+	double * pot_args,
+    tfuncs_type_arr pot_tfuncs,
+	double dpsi,
+	double rtol,
+	double atol,
+	double *result,
+	int * err,
+	int odeint_type,
+    orbint_callback_type cb){
+  //Set up the forces, first count
+  int ii,jj;
+  int dim;
+  int max_threads;
+  int * thread_pot_type;
+  double * thread_pot_args;
+  tfuncs_type_arr thread_pot_tfuncs;
+  max_threads= ( nobj < omp_get_max_threads() ) ? nobj : omp_get_max_threads();
+  // Because potentialArgs may cache, safest to have one / thread
+  struct potentialArg * potentialArgs= (struct potentialArg *) malloc ( max_threads * npot * sizeof (struct potentialArg) );
+#pragma omp parallel for schedule(static,1) private(ii,thread_pot_type,thread_pot_args,thread_pot_tfuncs) num_threads(max_threads)
+  for (ii=0; ii < max_threads; ii++) {
+    thread_pot_type= pot_type; // need to make thread-private pointers, bc
+    thread_pot_args= pot_args; // these pointers are changed in parse_...
+    thread_pot_tfuncs= pot_tfuncs; // ...
+    parse_leapFuncArgs(npot,potentialArgs+ii*npot,
+			    &thread_pot_type,&thread_pot_args,&thread_pot_tfuncs);
+  }
+  //Integrate
+  void (*odeint_func)(void (*func)(double, double *, double *,
+			   int, struct potentialArg *),
+		      int,
+		      double *,
+		      int, double, double *,
+		      int, struct potentialArg *,
+		      double, double,
+		      double *,int *);
+  void (*odeint_deriv_func)(double, double *, double *,
+			    int,struct potentialArg *);
+  dim= 5;
+  switch ( odeint_type ) {
+  // case 0: = leapfrog = not supported symplectic method
+  case 1: //RK4
+    odeint_func= &bovy_rk4;
+    break;
+  case 2: //RK6
+    odeint_func= &bovy_rk6;
+    break;
+  // case 3: = symplec4 = not supported symplectic method
+  // case 4: = symplec6 = not supported symplectic method
+  case 5: //DOPR54
+    odeint_func= &bovy_dopr54;
+    break;
+  case 6: //DOP853
+    odeint_func= &dop853;
+    break;
+  }
+  switch ( surface ) {
+    case 0: // x=0
+      odeint_deriv_func= &evalPlanarSOSDerivx;
+      break;
+    case 1: // y=0
+      odeint_deriv_func= &evalPlanarSOSDerivy;
+      break;
+  }
 
+#pragma omp parallel for schedule(dynamic,ORBITS_CHUNKSIZE) private(ii,jj) num_threads(max_threads)
+  for (ii=0; ii < nobj; ii++) {
+    polar_to_sos_galpy(yo+dim*ii,surface);
+    odeint_func(odeint_deriv_func,dim,yo+dim*ii,npsi,dpsi,psi+npsi*ii*indiv_psi,
+		npot,potentialArgs+omp_get_thread_num()*npot,rtol,atol,
+		result+dim*npsi*ii,err+ii);
+    for (jj=0; jj < npsi; jj++)
+      sos_to_polar_galpy(result+dim*jj+dim*npsi*ii,surface);
+    if ( cb ) // Callback if not void
+      cb();
+  }
+  //Free allocated memory
+#pragma omp parallel for schedule(static,1) private(ii) num_threads(max_threads)
+  for (ii=0; ii < max_threads; ii++)
+    free_potentialArgs(npot,potentialArgs+ii*npot);
+  free(potentialArgs);
+  //Done!
+}
 EXPORT void integratePlanarOrbit_dxdv(double *yo,
 				      int nt,
 				      double *t,
@@ -707,12 +869,12 @@ void evalPlanarRectForce(double t, double *q, double *a,
 }
 void evalPlanarRectDeriv(double t, double *q, double *a,
 			 int nargs, struct potentialArg * potentialArgs){
-  double sinphi, cosphi, x, y, phi,R,Rforce,phitorque;
+  double sinphi, cosphi, x, y, phi,R,Rforce,phitorque,vR,vT;
   //first two derivatives are just the velocities
   *a++= *(q+2);
   *a++= *(q+3);
   //Rest is force
-  //q is rectangular so calculate R and phi
+  //q is rectangular so calculate R and phi, vR and vT (for dissipative)
   x= *q;
   y= *(q+1);
   R= sqrt(x*x+y*y);
@@ -720,13 +882,83 @@ void evalPlanarRectDeriv(double t, double *q, double *a,
   sinphi= y/R;
   cosphi= x/R;
   if ( y < 0. ) phi= 2.*M_PI-phi;
+  vR=  *(q+2) * cosphi + *(q+3) * sinphi;
+  vT= -*(q+2) * sinphi + *(q+3) * cosphi;
   //Calculate the forces
-  Rforce= calcPlanarRforce(R,phi,t,nargs,potentialArgs);
-  phitorque= calcPlanarphitorque(R,phi,t,nargs,potentialArgs);
+  Rforce= calcPlanarRforce(R,phi,t,nargs,potentialArgs,vR,vT);
+  phitorque= calcPlanarphitorque(R,phi,t,nargs,potentialArgs,vR,vT);
   *a++= cosphi*Rforce-1./R*sinphi*phitorque;
   *a= sinphi*Rforce+1./R*cosphi*phitorque;
 }
 
+void evalPlanarSOSDerivx(double psi, double *q, double *a,
+		                 int nargs, struct potentialArg * potentialArgs){
+  // q= (y,vy,A,t,psi); to save operations, we reuse a first for the
+  // rectForce then for the actual RHS
+  // Note also that we keep track of psi in q+4, not in psi! This is
+  // such that we can avoid having to convert psi to psi+psi0
+  // q+4 starts as psi0 and then just increments as psi (exactly)
+  double sinpsi,cospsi,psidot,x,y,R,phi,sinphi,cosphi,Rforce,phitorque,vR,vT;
+  sinpsi= sin( *(q+4) );
+  cospsi= cos( *(q+4) );
+  // Calculate forces, put them in a+2, a+3
+  //q is rectangular so calculate R and phi
+  x= *(q+2) * sinpsi;
+  y= *(q  );
+  R= sqrt(x*x+y*y);
+  phi= atan2( y ,x );
+  sinphi= y/R;
+  cosphi= x/R;
+  vR=  *(q+2) * cospsi * cosphi + *(q+1) * sinphi;
+  vT= -*(q+2) * cospsi * sinphi + *(q+1) * cosphi;
+  //Calculate the forces
+  Rforce= calcPlanarRforce(R,phi,*(q+3),nargs,potentialArgs,vR,vT);
+  phitorque= calcPlanarphitorque(R,phi,*(q+3),nargs,potentialArgs,vR,vT);
+  *(a+2)= cosphi*Rforce-1./R*sinphi*phitorque;
+  *(a+3)= sinphi*Rforce+1./R*cosphi*phitorque;
+  // Now calculate the RHS of the ODE
+  psidot= cospsi * cospsi - sinpsi * *(a+2) / ( *(q+2) );
+  *(a  )= *(q+1) / psidot;
+  *(a+1)= *(a+3) / psidot;
+  *(a+2)= cospsi * ( *(q+2) * sinpsi + *(a+2) ) / psidot;
+  *(a+3)= 1./psidot;
+  *(a+4)= 1.; // dpsi / dpsi to keep track of psi
+}
+
+void evalPlanarSOSDerivy(double psi, double *q, double *a,
+		                 int nargs, struct potentialArg * potentialArgs){
+  // q= (x,vx,A,t,psi); to save operations, we reuse a first for the
+  // rectForce then for the actual RHS
+  // Note also that we keep track of psi in q+4, not in psi! This is
+  // such that we can avoid having to convert psi to psi+psi0
+  // q+4 starts as psi0 and then just increments as psi (exactly)
+  double sinpsi,cospsi,psidot,x,y,R,phi,sinphi,cosphi,Rforce,phitorque,vR,vT;
+  sinpsi= sin( *(q+4) );
+  cospsi= cos( *(q+4) );
+  // Calculate forces, put them in a+2, a+3
+  //q is rectangular so calculate R and phi
+  x= *(q  );
+  y= *(q+2) * sinpsi;
+  R= sqrt(x*x+y*y);
+  phi= atan2( y ,x );
+  sinphi= y/R;
+  cosphi= x/R;
+  vR=  *(q+1 ) * cosphi + *(q+2) * cospsi * sinphi;
+  vT= -*(q+1 ) * sinphi + *(q+2) * cospsi * cosphi;
+  //Calculate the forces
+  Rforce= calcPlanarRforce(R,phi,*(q+3),nargs,potentialArgs,vR,vT);
+  phitorque= calcPlanarphitorque(R,phi,*(q+3),nargs,potentialArgs,vR,vT);
+  *(a+2)= cosphi*Rforce-1./R*sinphi*phitorque;
+  *(a+3)= sinphi*Rforce+1./R*cosphi*phitorque;
+  // Now calculate the RHS of the ODE
+  psidot= cospsi * cospsi - sinpsi * *(a+3) / ( *(q+2) );
+  *(a  )= *(q+1) / psidot;
+  *(a+1)= *(a+2) / psidot;
+  *(a+2)= cospsi * ( *(q+2) * sinpsi + *(a+3) ) / psidot;
+  *(a+3)= 1./psidot;
+  *(a+4)= 1.; // dpsi / dpsi to keep track of psi
+}
+
 void evalPlanarRectDeriv_dxdv(double t, double *q, double *a,
 			      int nargs, struct potentialArg * potentialArgs){
   double sinphi, cosphi, x, y, phi,R,Rforce,phitorque;
