@@ -37,6 +37,7 @@ EXPORT void calc_potential(int nR,
 			   int npot,
 			   int * pot_type,
 			   double * pot_args,
+         tfuncs_type_arr pot_tfuncs,
 			   double *out,
 			   int * err){
   int ii, jj, tid, nthreads;
@@ -48,7 +49,7 @@ EXPORT void calc_potential(int nR,
   double * row= (double *) malloc ( nthreads * nz * ( sizeof ( double ) ) );
   //Set up the potentials
   struct potentialArg * potentialArgs= (struct potentialArg *) malloc ( npot * sizeof (struct potentialArg) );
-  parse_leapFuncArgs_Full(npot,potentialArgs,&pot_type,&pot_args);
+  parse_leapFuncArgs_Full(npot,potentialArgs,&pot_type,&pot_args,&pot_tfuncs);
   //Run through the grid and calculate
   UNUSED int chunk= CHUNKSIZE;
 #pragma omp parallel for schedule(static,chunk) private(ii,tid,jj)	\
@@ -62,7 +63,7 @@ EXPORT void calc_potential(int nR,
     for (jj=0; jj < nz; jj++){
       *(row+jj+tid*nz)= evaluatePotentials(*(R+ii),*(z+jj),npot,potentialArgs);
     }
-    put_row(out,ii,row+tid*nz,nz); 
+    put_row(out,ii,row+tid*nz,nz);
   }
   free_potentialArgs(npot,potentialArgs);
   free(potentialArgs);
@@ -75,6 +76,7 @@ EXPORT void calc_rforce(int nR,
 			int npot,
 			int * pot_type,
 			double * pot_args,
+      tfuncs_type_arr pot_tfuncs,
 			double *out,
 			int * err){
   int ii, jj, tid, nthreads;
@@ -86,7 +88,7 @@ EXPORT void calc_rforce(int nR,
   double * row= (double *) malloc ( nthreads * nz * ( sizeof ( double ) ) );
   //Set up the potentials
   struct potentialArg * potentialArgs= (struct potentialArg *) malloc ( npot * sizeof (struct potentialArg) );
-  parse_leapFuncArgs_Full(npot,potentialArgs,&pot_type,&pot_args);
+  parse_leapFuncArgs_Full(npot,potentialArgs,&pot_type,&pot_args,&pot_tfuncs);
   //Run through the grid and calculate
   UNUSED int chunk= CHUNKSIZE;
 #pragma omp parallel for schedule(static,chunk) private(ii,tid,jj)	\
@@ -100,7 +102,7 @@ EXPORT void calc_rforce(int nR,
     for (jj=0; jj < nz; jj++){
       *(row+jj+tid*nz)= calcRforce(*(R+ii),*(z+jj),0.,0.,npot,potentialArgs);
     }
-    put_row(out,ii,row+tid*nz,nz); 
+    put_row(out,ii,row+tid*nz,nz);
   }
   free_potentialArgs(npot,potentialArgs);
   free(potentialArgs);
@@ -113,6 +115,7 @@ EXPORT void calc_zforce(int nR,
 			int npot,
 			int * pot_type,
 			double * pot_args,
+      tfuncs_type_arr pot_tfuncs,
 			double *out,
 			int * err){
   int ii, jj, tid, nthreads;
@@ -124,7 +127,7 @@ EXPORT void calc_zforce(int nR,
   double * row= (double *) malloc ( nthreads * nz * ( sizeof ( double ) ) );
   //Set up the potentials
   struct potentialArg * potentialArgs= (struct potentialArg *) malloc ( npot * sizeof (struct potentialArg) );
-  parse_leapFuncArgs_Full(npot,potentialArgs,&pot_type,&pot_args);
+  parse_leapFuncArgs_Full(npot,potentialArgs,&pot_type,&pot_args,&pot_tfuncs);
   //Run through the grid and calculate
   UNUSED int chunk= CHUNKSIZE;
 #pragma omp parallel for schedule(static,chunk) private(ii,tid,jj)	\
@@ -138,7 +141,7 @@ EXPORT void calc_zforce(int nR,
     for (jj=0; jj < nz; jj++){
       *(row+jj+tid*nz)= calczforce(*(R+ii),*(z+jj),0.,0.,npot,potentialArgs);
     }
-    put_row(out,ii,row+tid*nz,nz); 
+    put_row(out,ii,row+tid*nz,nz);
   }
   free_potentialArgs(npot,potentialArgs);
   free(potentialArgs);
@@ -150,12 +153,13 @@ EXPORT void eval_potential(int nR,
 			   int npot,
 			   int * pot_type,
 			   double * pot_args,
+         tfuncs_type_arr pot_tfuncs,
 			   double *out,
 			   int * err){
   int ii;
   //Set up the potentials
   struct potentialArg * potentialArgs= (struct potentialArg *) malloc ( npot * sizeof (struct potentialArg) );
-  parse_leapFuncArgs_Full(npot,potentialArgs,&pot_type,&pot_args);
+  parse_leapFuncArgs_Full(npot,potentialArgs,&pot_type,&pot_args,&pot_tfuncs);
   //Run through and evaluate
   for (ii=0; ii < nR; ii++){
     *(out+ii)= evaluatePotentials(*(R+ii),*(z+ii),npot,potentialArgs);
@@ -169,12 +173,13 @@ EXPORT void eval_rforce(int nR,
 			int npot,
 			int * pot_type,
 			double * pot_args,
+      tfuncs_type_arr pot_tfuncs,
 			double *out,
 			int * err){
   int ii;
   //Set up the potentials
   struct potentialArg * potentialArgs= (struct potentialArg *) malloc ( npot * sizeof (struct potentialArg) );
-  parse_leapFuncArgs_Full(npot,potentialArgs,&pot_type,&pot_args);
+  parse_leapFuncArgs_Full(npot,potentialArgs,&pot_type,&pot_args,&pot_tfuncs);
   //Run through and evaluate
   for (ii=0; ii < nR; ii++){
     *(out+ii)= calcRforce(*(R+ii),*(z+ii),0.,0.,npot,potentialArgs);
@@ -188,12 +193,13 @@ EXPORT void eval_zforce(int nR,
 			int npot,
 			int * pot_type,
 			double * pot_args,
+      tfuncs_type_arr pot_tfuncs,
 			double *out,
 			int * err){
   int ii;
   //Set up the potentials
   struct potentialArg * potentialArgs= (struct potentialArg *) malloc ( npot * sizeof (struct potentialArg) );
-  parse_leapFuncArgs_Full(npot,potentialArgs,&pot_type,&pot_args);
+  parse_leapFuncArgs_Full(npot,potentialArgs,&pot_type,&pot_args,&pot_tfuncs);
   //Run through and evaluate
   for (ii=0; ii < nR; ii++){
     *(out+ii)= calczforce(*(R+ii),*(z+ii),0.,0.,npot,potentialArgs);
