@@ -114,7 +114,7 @@ void NonInertialFrameForcexyzforces_xyz(double R,double z,double phi,double t,
         v0x= (*(*(potentialArgs->tfuncs+6)))(t);
         v0y= (*(*(potentialArgs->tfuncs+7)))(t);
         v0z= (*(*(potentialArgs->tfuncs+8)))(t);
-        // Re-use variable
+        // Reuse variable
         Omegatimesvecx= Omegax * x0x + Omegay * x0y + Omegaz * x0z;
         *Fx+=  2. * ( Omegaz * v0y - Omegay * v0z ) + Omega2 * x0x - Omegax * Omegatimesvecx;
         *Fy+= -2. * ( Omegaz * v0x - Omegax * v0z ) + Omega2 * x0y - Omegay * Omegatimesvecx;
@@ -166,6 +166,11 @@ double NonInertialFrameForceRforce(double R,double z,double phi,double t,
   }
   return amp * ( cos ( phi ) * Fx + sin( phi ) * Fy );
 }
+double NonInertialFrameForcePlanarRforce(double R,double phi,double t,
+				                         struct potentialArg * potentialArgs,
+                                         double vR,double vT){
+  return NonInertialFrameForceRforce(R,0.,phi,t,potentialArgs,vR,vT,0.);
+}
 double NonInertialFrameForcephitorque(double R,double z,double phi,double t,
 				                             struct potentialArg * potentialArgs,
                                      double vR,double vT,double vz){
@@ -194,6 +199,11 @@ double NonInertialFrameForcephitorque(double R,double z,double phi,double t,
   }
   return amp * R * ( -sin ( phi ) * Fx + cos( phi ) * Fy );
 }
+double NonInertialFrameForcePlanarphitorque(double R,double phi,double t,
+				                            struct potentialArg * potentialArgs,
+                                            double vR,double vT){
+  return NonInertialFrameForcephitorque(R,0.,phi,t,potentialArgs,vR,vT,0.);
+}
 double NonInertialFrameForcezforce(double R,double z,double phi,double t,
 				                           struct potentialArg * potentialArgs,
                                    double vR,double vT,double vz){
