@@ -147,9 +147,10 @@ void leapfrog(void (*func)(double t, double *q, double *a,
       *err= -10;
       interrupted= 0; // need to reset, bc library and vars stay in memory
 #ifdef USING_COVERAGE
-      __gcov_flush();
-#endif
+      __gcov_dump();
 // LCOV_EXCL_START
+      __gcov_reset();
+#endif
       break;
 // LCOV_EXCL_STOP
     }
@@ -272,9 +273,10 @@ void symplec4(void (*func)(double t, double *q, double *a,
       *err= -10;
       interrupted= 0; // need to reset, bc library and vars stay in memory
 #ifdef USING_COVERAGE
-      __gcov_flush();
-#endif
+      __gcov_dump();
 // LCOV_EXCL_START
+      __gcov_reset();
+#endif
       break;
 // LCOV_EXCL_STOP
     }
@@ -432,9 +434,10 @@ void symplec6(void (*func)(double t, double *q, double *a,
       *err= -10;
       interrupted= 0; // need to reset, bc library and vars stay in memory
 #ifdef USING_COVERAGE
-      __gcov_flush();
-#endif
+      __gcov_dump();
 // LCOV_EXCL_START
+      __gcov_reset();
+#endif
       break;
 // LCOV_EXCL_STOP
     }
