@@ -139,60 +139,60 @@ bhh2 = 0.733846688281611857341361741547
 bhh3 = 0.220588235294117647058823529412e-1
 
 # Dense output coefficients
-d41 = -0.84289382761090128651353491142e+1
+d41 = -0.84289382761090128651353491142e1
 d46 = 0.56671495351937776962531783590
-d47 = -0.30689499459498916912797304727e+1
-d48 = 0.23846676565120698287728149680e+1
-d49 = 0.21170345824450282767155149946e+1
+d47 = -0.30689499459498916912797304727e1
+d48 = 0.23846676565120698287728149680e1
+d49 = 0.21170345824450282767155149946e1
 d410 = -0.87139158377797299206789907490
-d411 = 0.22404374302607882758541771650e+1
+d411 = 0.22404374302607882758541771650e1
 d412 = 0.63157877876946881815570249290
 d413 = -0.88990336451333310820698117400e-1
-d414 = 0.18148505520854727256656404962e+2
-d415 = -0.91946323924783554000451984436e+1
-d416 = -0.44360363875948939664310572000e+1
-d51 = 0.10427508642579134603413151009e+2
-d56 = 0.24228349177525818288430175319e+3
-d57 = 0.16520045171727028198505394887e+3
-d58 = -0.37454675472269020279518312152e+3
-d59 = -0.22113666853125306036270938578e+2
-d510 = 0.77334326684722638389603898808e+1
-d511 = -0.30674084731089398182061213626e+2
-d512 = -0.93321305264302278729567221706e+1
-d513 = 0.15697238121770843886131091075e+2
-d514 = -0.31139403219565177677282850411e+2
-d515 = -0.93529243588444783865713862664e+1
-d516 = 0.35816841486394083752465898540e+2
-d61 = 0.19985053242002433820987653617e+2
-d66 = -0.38703730874935176555105901742e+3
-d67 = -0.18917813819516756882830838328e+3
-d68 = 0.52780815920542364900561016686e+3
-d69 = -0.11573902539959630126141871134e+2
-d610 = 0.68812326946963000169666922661e+1
-d611 = -0.10006050966910838403183860980e+1
+d414 = 0.18148505520854727256656404962e2
+d415 = -0.91946323924783554000451984436e1
+d416 = -0.44360363875948939664310572000e1
+d51 = 0.10427508642579134603413151009e2
+d56 = 0.24228349177525818288430175319e3
+d57 = 0.16520045171727028198505394887e3
+d58 = -0.37454675472269020279518312152e3
+d59 = -0.22113666853125306036270938578e2
+d510 = 0.77334326684722638389603898808e1
+d511 = -0.30674084731089398182061213626e2
+d512 = -0.93321305264302278729567221706e1
+d513 = 0.15697238121770843886131091075e2
+d514 = -0.31139403219565177677282850411e2
+d515 = -0.93529243588444783865713862664e1
+d516 = 0.35816841486394083752465898540e2
+d61 = 0.19985053242002433820987653617e2
+d66 = -0.38703730874935176555105901742e3
+d67 = -0.18917813819516756882830838328e3
+d68 = 0.52780815920542364900561016686e3
+d69 = -0.11573902539959630126141871134e2
+d610 = 0.68812326946963000169666922661e1
+d611 = -0.10006050966910838403183860980e1
 d612 = 0.77771377980534432092869265740
-d613 = -0.27782057523535084065932004339e+1
-d614 = -0.60196695231264120758267380846e+2
-d615 = 0.84320405506677161018159903784e+2
-d616 = 0.11992291136182789328035130030e+2
-d71 = -0.25693933462703749003312586129e+2
-d76 = -0.15418974869023643374053993627e+3
-d77 = -0.23152937917604549567536039109e+3
-d78 = 0.35763911791061412378285349910e+3
-d79 = 0.93405324183624310003907691704e+2
-d710 = -0.37458323136451633156875139351e+2
-d711 = 0.10409964950896230045147246184e+3
-d712 = 0.29840293426660503123344363579e+2
-d713 = -0.43533456590011143754432175058e+2
-d714 = 0.96324553959188282948394950600e+2
-d715 = -0.39177261675615439165231486172e+2
-d716 = -0.14972683625798562581422125276e+3
+d613 = -0.27782057523535084065932004339e1
+d614 = -0.60196695231264120758267380846e2
+d615 = 0.84320405506677161018159903784e2
+d616 = 0.11992291136182789328035130030e2
+d71 = -0.25693933462703749003312586129e2
+d76 = -0.15418974869023643374053993627e3
+d77 = -0.23152937917604549567536039109e3
+d78 = 0.35763911791061412378285349910e3
+d79 = 0.93405324183624310003907691704e2
+d710 = -0.37458323136451633156875139351e2
+d711 = 0.10409964950896230045147246184e3
+d712 = 0.29840293426660503123344363579e2
+d713 = -0.43533456590011143754432175058e2
+d714 = 0.96324553959188282948394950600e2
+d715 = -0.39177261675615439165231486172e2
+d716 = -0.14972683625798562581422125276e3
 
 # Error calculation coefficients
 er1 = 0.1312004499419488073250102996e-1
-er6 = -0.1225156446376204440720569753e+1
+er6 = -0.1225156446376204440720569753e1
 er7 = -0.4957589496572501915214079952
-er8 = 0.1664377182454986536961530415e+1
+er8 = 0.1664377182454986536961530415e1
 er9 = -0.3503288487499736816886487290
 er10 = 0.3341791187130174790297318841
 er11 = 0.8192320648511571246570742613e-1
@@ -200,7 +200,7 @@ er12 = -0.2235530786388629525884427845e-1
 
 # machine limit related info from numpy
 unsigned_int_max = numpy.iinfo(numpy.int32).max
-uround = numpy.finfo(numpy.float).eps
+uround = numpy.finfo(numpy.float64).eps
 
 
 def custom_sign(a, b):
@@ -232,24 +232,38 @@ def hinit(func, x, t, pos_neg, f0, iord, hmax, rtol, atol, args):
     der12 = numpy.max([numpy.fabs(der2), numpy.sqrt(dnf)])
     h1 = numpy.power(0.01 / der12, 1.0 / iord)
 
-    h = numpy.min([100.0 * numpy.fabs(h), numpy.min([numpy.fabs(h1), numpy.fabs(hmax)])])
+    h = numpy.min(
+        [100.0 * numpy.fabs(h), numpy.min([numpy.fabs(h1), numpy.fabs(hmax)])]
+    )
 
     return custom_sign(h, pos_neg), f0, f1, xx1
 
 
 def dense_output(t_current, t_old, h_current, rcont):
     """
-    Dense output function, basically extrapolatin
+    Dense output function, basically extrapolating
     """
     # initialization
     s = (t_current - t_old) / h_current
     s1 = 1.0 - s
 
-    return rcont[0] + s * (rcont[1] + s1 * (
-            rcont[2] + s * (rcont[3] + s1 * (rcont[4] + s * (rcont[5] + s1 * (rcont[6] + s * rcont[7]))))))
+    return rcont[0] + s * (
+        rcont[1]
+        + s1
+        * (
+            rcont[2]
+            + s
+            * (
+                rcont[3]
+                + s1 * (rcont[4] + s * (rcont[5] + s1 * (rcont[6] + s * rcont[7])))
+            )
+        )
+    )
 
 
-def dopri853core(n, func, x, t, hmax, h, rtol, atol, nmax, safe, beta, fac1, fac2, pos_neg, args):
+def dopri853core(
+    n, func, x, t, hmax, h, rtol, atol, nmax, safe, beta, fac1, fac2, pos_neg, args
+):
     """
     Core of DOP8(5, 3) integration
     """
@@ -270,14 +284,18 @@ def dopri853core(n, func, x, t, hmax, h, rtol, atol, nmax, safe, beta, fac1, fac
         h, k1, k2, k3 = hinit(func, x, t, pos_neg, k1, iord, hmax, rtol, atol, args)
 
     reject = 0
-    t_current = t[0]  # store current integration time internally (not the current time wanted by user!!)
+    t_current = t[
+        0
+    ]  # store current integration time internally (not the current time wanted by user!!)
     t_old = t[0]
     finished_user_t_ii = 0  # times indices wanted by user
 
     result[0, :] = x
 
     # basic integration step
-    while finished_user_t_ii < len(t) - 1:  # check if the current computed time indices less than total inices needed
+    while (
+        finished_user_t_ii < len(t) - 1
+    ):  # check if the current computed time indices less than total inices needed
         # keep time step not too small
         h = pos_neg * numpy.max([numpy.fabs(h), 1e3 * uround])
 
@@ -306,14 +324,40 @@ def dopri853core(n, func, x, t, hmax, h, rtol, atol, nmax, safe, beta, fac1, fac
         xx1 = x + h * (a91 * k1 + a94 * k4 + a95 * k5 + a96 * k6 + a97 * k7 + a98 * k8)
         k9 = numpy.array(func(xx1, t_current + c9 * h, *args))
 
-        xx1 = x + h * (a101 * k1 + a104 * k4 + a105 * k5 + a106 * k6 + a107 * k7 + a108 * k8 + a109 * k9)
+        xx1 = x + h * (
+            a101 * k1
+            + a104 * k4
+            + a105 * k5
+            + a106 * k6
+            + a107 * k7
+            + a108 * k8
+            + a109 * k9
+        )
         k10 = numpy.array(func(xx1, t_current + c10 * h, *args))
 
-        xx1 = x + h * (a111 * k1 + a114 * k4 + a115 * k5 + a116 * k6 + a117 * k7 + a118 * k8 + a119 * k9 + a1110 * k10)
+        xx1 = x + h * (
+            a111 * k1
+            + a114 * k4
+            + a115 * k5
+            + a116 * k6
+            + a117 * k7
+            + a118 * k8
+            + a119 * k9
+            + a1110 * k10
+        )
         k2 = numpy.array(func(xx1, t_current + c11 * h, *args))
 
         xx1 = x + h * (
-                a121 * k1 + a124 * k4 + a125 * k5 + a126 * k6 + a127 * k7 + a128 * k8 + a129 * k9 + a1210 * k10 + a1211 * k2)
+            a121 * k1
+            + a124 * k4
+            + a125 * k5
+            + a126 * k6
+            + a127 * k7
+            + a128 * k8
+            + a129 * k9
+            + a1210 * k10
+            + a1211 * k2
+        )
 
         t_old_older = numpy.copy(t_old)
         t_old = numpy.copy(t_current)
@@ -321,14 +365,32 @@ def dopri853core(n, func, x, t, hmax, h, rtol, atol, nmax, safe, beta, fac1, fac
 
         k3 = numpy.array(func(xx1, t_current, *args))
 
-        k4 = b1 * k1 + b6 * k6 + b7 * k7 + b8 * k8 + b9 * k9 + b10 * k10 + b11 * k2 + b12 * k3
+        k4 = (
+            b1 * k1
+            + b6 * k6
+            + b7 * k7
+            + b8 * k8
+            + b9 * k9
+            + b10 * k10
+            + b11 * k2
+            + b12 * k3
+        )
         k5 = x + h * k4
 
         # error estimation
         sk = atol + rtol * numpy.max([numpy.fabs(x), numpy.fabs(k5)], axis=0)
         erri = k4 - bhh1 * k1 - bhh2 * k9 - bhh3 * k3
         err2 = numpy.sum(numpy.square(erri / sk), axis=0)
-        erri = er1 * k1 + er6 * k6 + er7 * k7 + er8 * k8 + er9 * k9 + er10 * k10 + er11 * k2 + er12 * k3
+        erri = (
+            er1 * k1
+            + er6 * k6
+            + er7 * k7
+            + er8 * k8
+            + er9 * k9
+            + er10 * k10
+            + er11 * k2
+            + er12 * k3
+        )
         err = numpy.sum(numpy.square(erri / sk), axis=0)
 
         deno = err + 0.01 * err2
@@ -357,20 +419,80 @@ def dopri853core(n, func, x, t, hmax, h, rtol, atol, nmax, safe, beta, fac1, fac
             bspl = h * k1 - xdiff
             rcont3 = numpy.copy(bspl)
             rcont4 = xdiff - h * k4 - bspl
-            rcont5 = d41 * k1 + d46 * k6 + d47 * k7 + d48 * k8 + d49 * k9 + d410 * k10 + d411 * k2 + d412 * k3
-            rcont6 = d51 * k1 + d56 * k6 + d57 * k7 + d58 * k8 + d59 * k9 + d510 * k10 + d511 * k2 + d512 * k3
-            rcont7 = d61 * k1 + d66 * k6 + d67 * k7 + d68 * k8 + d69 * k9 + d610 * k10 + d611 * k2 + d612 * k3
-            rcont8 = d71 * k1 + d76 * k6 + d77 * k7 + d78 * k8 + d79 * k9 + d710 * k10 + d711 * k2 + d712 * k3
+            rcont5 = (
+                d41 * k1
+                + d46 * k6
+                + d47 * k7
+                + d48 * k8
+                + d49 * k9
+                + d410 * k10
+                + d411 * k2
+                + d412 * k3
+            )
+            rcont6 = (
+                d51 * k1
+                + d56 * k6
+                + d57 * k7
+                + d58 * k8
+                + d59 * k9
+                + d510 * k10
+                + d511 * k2
+                + d512 * k3
+            )
+            rcont7 = (
+                d61 * k1
+                + d66 * k6
+                + d67 * k7
+                + d68 * k8
+                + d69 * k9
+                + d610 * k10
+                + d611 * k2
+                + d612 * k3
+            )
+            rcont8 = (
+                d71 * k1
+                + d76 * k6
+                + d77 * k7
+                + d78 * k8
+                + d79 * k9
+                + d710 * k10
+                + d711 * k2
+                + d712 * k3
+            )
 
             # the next three function evaluations
             xx1 = x + h * (
-                    a141 * k1 + a147 * k7 + a148 * k8 + a149 * k9 + a1410 * k10 + a1411 * k2 + a1412 * k3 + a1413 * k4)
+                a141 * k1
+                + a147 * k7
+                + a148 * k8
+                + a149 * k9
+                + a1410 * k10
+                + a1411 * k2
+                + a1412 * k3
+                + a1413 * k4
+            )
             k10 = numpy.array(func(xx1, t_old + c14 * h, *args))
             xx1 = x + h * (
-                    a151 * k1 + a156 * k6 + a157 * k7 + a158 * k8 + a1511 * k2 + a1512 * k3 + a1513 * k4 + a1514 * k10)
+                a151 * k1
+                + a156 * k6
+                + a157 * k7
+                + a158 * k8
+                + a1511 * k2
+                + a1512 * k3
+                + a1513 * k4
+                + a1514 * k10
+            )
             k2 = numpy.array(func(xx1, t_old + c15 * h, *args))
             xx1 = x + h * (
-                    a161 * k1 + a166 * k6 + a167 * k7 + a168 * k8 + a169 * k9 + a1613 * k4 + a1614 * k10 + a1615 * k2)
+                a161 * k1
+                + a166 * k6
+                + a167 * k7
+                + a168 * k8
+                + a169 * k9
+                + a1613 * k4
+                + a1614 * k10
+                + a1615 * k2
+            )
             k3 = numpy.array(func(xx1, t_old + c16 * h, *args))
 
             # final preparation
@@ -383,11 +505,15 @@ def dopri853core(n, func, x, t, hmax, h, rtol, atol, nmax, safe, beta, fac1, fac
             x = numpy.copy(k5)
 
             # loop for dense output in this time slot
-            while (finished_user_t_ii < len(t) - 1) and (pos_neg * t[finished_user_t_ii + 1] < pos_neg * t_current):
-                result[finished_user_t_ii + 1, :] = dense_output(t[finished_user_t_ii + 1], t_old, h,
-                                                                 [rcont1, rcont2, rcont3, rcont4, rcont5, rcont6,
-                                                                  rcont7,
-                                                                  rcont8])
+            while (finished_user_t_ii < len(t) - 1) and (
+                pos_neg * t[finished_user_t_ii + 1] < pos_neg * t_current
+            ):
+                result[finished_user_t_ii + 1, :] = dense_output(
+                    t[finished_user_t_ii + 1],
+                    t_old,
+                    h,
+                    [rcont1, rcont2, rcont3, rcont4, rcont5, rcont6, rcont7, rcont8],
+                )
                 finished_user_t_ii += 1
 
             if numpy.fabs(hnew) > hmax:
@@ -410,55 +536,61 @@ def dopri853core(n, func, x, t, hmax, h, rtol, atol, nmax, safe, beta, fac1, fac
     return result
 
 
-def dop853(func=None,
-           x=None,
-           t=None,
-           hmax=0.,
-           h=0.,
-           rtol=1e-12,
-           atol=1e-12,
-           nmax=int(1e8),
-           safe=0.9,
-           beta=0.,
-           fac1=0.333,
-           fac2=6.,
-           args=()
-           ):
+def dop853(
+    func=None,
+    x=None,
+    t=None,
+    hmax=0.0,
+    h=0.0,
+    rtol=1e-12,
+    atol=1e-12,
+    nmax=int(1e8),
+    safe=0.9,
+    beta=0.0,
+    fac1=0.333,
+    fac2=6.0,
+    args=(),
+):
     """
-    To computes the numerical solution of a system of first order ordinary differential equations y'=f(x,y).
-    It uses an explicit Runge-Kutta method of order 8(5,3) due to Dormand & Prince with step size control and dense output.
+    Solve a system of ordinary differential equations using the DOP853 method, an explicit Runge-Kutta method of order 8(5,3) due to Dormand & Prince with step size control and dense output.
 
-    Reference: E. Hairer, S.P. Norsett and G. Wanner, Solving ordinary differential equations I,
-    non-stiff problems, 2nd edition, Springer Series in Computational Mathematics, Springer-Verlag (1993).
+    Parameters
+    ----------
+    func : callable
+        function of the differential equation, usually take func([position, velocity], time) and return velocity, acceleration
+    x : float or ndarray
+        initial x, usually is [position, velocity]
+    t : ndarray
+        set of times at which one wants the result
+    hmax : float
+        Maximal step size, default 0. which will be internally as t[-1]-t[0]
+    h : float
+        Initial step size, default 0. which will be computed by the function hinit()
+    rtol : float or ndarray
+        Relative error tolerances, default 1e-8
+    atol : float or ndarray
+        Absolute error tolerances, default 1e-8
+    nmax : int
+        Maximum number of steps, default 1e8
+    safe : float
+        Safety factor in the step size prediction, default 0.9
+    beta : float
+        The "beta" for stabilized step size control (see section IV.2 of the reference). Larger values for beta (e.g.<= 0.1) make the step size control more stable. Default beta=0.
+    fac1, fac2 : float
+        Parameters for step size selection; the new step size is chosen subject to the restriction fac1 <= hnew/hold <= fac2
 
-    :param func: function of the differential equation, usually take func([position, velocity], time) and return velocity, acceleration
-    :type func: callable
-    :param x: initial x, usually is [position, velocity]
-    :type x: Union[float, ndarray]
-    :param t: set of times at which one wants the result
-    :type t: ndarray
-    :param hmax: Maximal step size, default 0. which will be internally as t[-1]-t[0]
-    :type hmax: float
-    :param h: Initial step size, default 0. which will be computed by the function hinit()
-    :type h: float
-    :param rtol: Relative error tolerances, default 1e-8
-    :type rtol: Union[float, ndarray]
-    :param atol: Absolute error tolerances, default 1e-8
-    :type atol: Union[float, ndarray]
-    :param nmax: Maximum number of steps, default 1e8
-    :type nmax: int
-    :param safe: Safety factor in the step size prediction, default 0.9
-    :type atol: float
-    :param beta: | The "beta" for stabilized step size control (see section IV.2 of the reference). Larger values for beta
-                 | (e.g.<= 0.1) make the step size control more stable. Default beta=0.
-    :type beta: float
-    :param fac1, fac2: Parameters for step size selection; the new step size is chosen subject to the restriction fac1 <= hnew/hold <= fac2
-    :type fac1, fac2: float
+    Returns
+    -------
+    ndarray
+        integrated result
 
-    :return: integrated result
-    :rtype: ndarray
+    Notes
+    -----
+    - 2018-11-23 - Written - Henry Leung (University of Toronto)
 
-    :History: 2018-Nov-23 - Written - Henry Leung (University of Toronto)
+    References
+    ----------
+    - E. Hairer, S.P. Norsett and G. Wanner, Solving ordinary differential equations I, non-stiff problems, 2nd edition, Springer Series in Computational Mathematics, Springer-Verlag (1993).
     """
     # initialization
     n = len(x)
@@ -468,8 +600,10 @@ def dop853(func=None,
         hmax = t[-1] - t[0]
 
     # see if integrate forward or integrate backward
-    pos_neg = custom_sign(1., t[-1] - t[0])
+    pos_neg = custom_sign(1.0, t[-1] - t[0])
 
-    result = dopri853core(n, func, x, t, hmax, h, rtol, atol, nmax, safe, beta, fac1, fac2, pos_neg, args)
+    result = dopri853core(
+        n, func, x, t, hmax, h, rtol, atol, nmax, safe, beta, fac1, fac2, pos_neg, args
+    )
 
     return result
